;**************************************************
 pro tile_to_pos,nx,ny,pos,psize=psize,fsize=fsize,page=page,posx,posy
;**************************************************
; Procedure for defining giving positions
; for plotting, returned in pos(nx*ny,4) 
;**************************************************

; variables

; nx =number of panels in x direction (0-> square plot including > ny windows)
; ny =number of panels in y direction
; psize =size of page relative to 0.6*0.8
; fsize =size of frame relative to 0.6/nx * 0.8/ny

if(n_params() eq 0) then begin
print,' tile_to_pos,nx,ny,pos,posx,posy,'
print,'             psize=psize,fsize=fsize,page=page'
print,' divides page into nx*ny square-shaped panels'
print,' pos(nx*ny,4) returns positions of individual frames'
print,' /psize determines relative pagesize (default=1 -> 0.6*0.8)'
print,' /fsize determines relative framesize (default=1)'
print,' if non-square region is required:
print,' /page -> total size of page=[x1,y1,x2,y2]'
print,'---------------'
print,' for TV-plots:'
print,' give window size in page
print,' posx(nx*ny) returns left-x position of individual frames
print,' posy(nx*ny) returns down-y position of individual frames'
return
endif


pos=fltarr(nx*ny,4)
posx=fltarr(nx*ny)
posy=fltarr(nx*ny)
if not keyword_set(psize) then psize=1.
if not keyword_set(fsize) then fsize=1.

;***********************************
; square-shaped region
;***********************************
if(not keyword_set(page)) then begin

;page center
xc=0.5
yc=0.5

; page size
if(ny le nx ) then begin
pagex=0.6*psize
pagey=0.8*psize*ny/nx
endif
if(ny gt nx) then begin
pagey=0.8*psize
pagex=0.6*psize*nx/ny
endif

; frame size
framex=pagex/nx*fsize
framey=pagey/ny*fsize
endif

;***********************************
; non-square-shaped region
;***********************************
if(keyword_set(page)) then begin

xx1=page(0)
xx2=page(2)
yy1=page(1)
yy2=page(3)

;page center
xc=(xx1+xx2)/2.
yc=(yy1+yy2)/2.

; page size
pagex=(xx2-xx1)*psize
pagey=(yy2-yy1)*psize


; frame size
framex=pagex/nx*fsize
framey=pagey/ny*fsize
endif


for nf=0,nx*ny-1 do begin
; center of frame nf
iy=fix((nf)/nx)+1
ix=(nf+1)-nx*(iy-1)

; set wievport
; x1=page-edge + distance to frame center - framesize/2
x1=(xc-pagex/2.)+pagex/nx*(ix-.5)-framex/2.
x2=x1+framex
y2=(yc+pagey/2.)-pagey/ny*(iy-.5)+framey/2.
y1=y2-framey

posx(nf)=x1
posy(nf)=y1

pos(nf,0:3)=[x1,y1,x2,y2]

endfor
end



















